#! /usr/bin/perl -w

#--------------------------------------- subroutines
sub writeSpiC;
sub writeSpiM;

#--------------------------------------- Perl initialization
$|=1;
$OSName=`uname -s`;
chomp($OSName);
# $nsec=512;

$debug=0;

#--------------------------------------- Set environment variables
$prefix="/usr";
$exec_prefix="${prefix}";
$pkgdatadir="${prefix}/share/gds";
$ENV{ROOTSYS} = "/usr" if (!defined($ENV{ROOTSYS}));
$gdspath="$exec_prefix";
$rootpath="$ENV{ROOTSYS}";
$ENV{PATH} = "${exec_prefix}/bin:$rootpath/bin:/usr/bin:/bin";

#---------------------------------------  Get thhe user name
$user=`whoami`;
chomp($user);

#---------------------------------------  Commands
$SMDUMP=`which smdump`;
chomp($SMDUMP);
$TOP=`which top`;
chomp($TOP);
$WGET="wget -t 1 -T 5 -q --no-check-certificate";
$WHEN=`which when`;
chomp($WHEN);

$TNOW="$WHEN 0 %s";
if ( "$OSName" eq "SunOS" ) {
    $TOPCMD="$TOP -d1 -s1 -b 1000";
} else {
    $TOPCMD="$TOP -b -d1 -n1 -u$user";
}

#---------------------------------------  Save time stamp
$T_start=`$TNOW`;

#---------------------------------------  Location of stuff
#
#  sd     Spi local cache directory
#  td     Spi Parameter directories
#  TD     Temporary directory
#  opsdir Monitor run directories.
#
$sd="$ENV{HOME}/dmtspi/cache";
$td="$pkgdatadir/Spi";
system("mkdir -p $sd");
$TD="/tmp";
$opsdir=`bash -c 'echo ~$user'`;
chomp($opsdir);

#---------------------------------------  Identify machine
#
#  gn  contains the node name of the current (hostname)
#  g   contains the host nickname (e.g. STONE rather than gdsdmt1)
#  gip contains the site domain (e.g. ligo-wa or ligo-la)
#  gLN contains the observatory name (e.g. LHO or LLO)
#
@hostlist=`cat $td/HOST.lookup`;
chomp(@hostlist);
$gn=`uname -n`;
chomp($gn);
@idline=grep(/^$gn /, @hostlist);
if ( ! @idline ) {
    $g=$gn;
    $gip="ligo";
    $gLN="CIT";
    $dh="$ENV{HOME}/public_html/gds";
    $dw="http://www.ligo.caltech.edu/~$user/gds";
    $ms="M";
} else {
    @idlist=split(/[ \t]+/, $idline[0]);
    $g=$idlist[1];
    $gip=$idlist[2];
    $gLN=$idlist[3];
    $dw=$idlist[4];
    $ms=$idlist[5];
}

#----------------------------------------   See if the partition exists
$ENV{LIGOSMPART}="${gLN}_Online" if (!defined($ENV{LIGOSMPART}));
$test_lsmp = `$gdspath/bin/smstat exists $ENV{LIGOSMPART}`;
chomp($test_lsmp);

#----------------------------------------   Define remote directories
if ("$gip" eq "ligo-wa") {
    $dh="/gds-h1/dmt/html";
    $dw="https://marble.ligo-wa.caltech.edu/dmt";
} elsif ("$gip" eq "ligo-la") {
    $dh="/gds/dmt/html";
    $dw="https://llodmt.ligo-la.caltech.edu/dmt";
}

#----------------------------------------  Monitor documentation path and url
$dd="$dh/Monitors";
$d2="$dw/Monitors";

#----------------------------------------  Spi page path and url
$wdir="$dh/Monitors";
$wurl="$dw/Monitors";

unlink("$sd/${g}_Errors.log");

$IsSluggish="NO";

#----------------------------------------  Get consumer list.
mkdir("$wdir/$g");
if ( $test_lsmp != 0 ) {
    $statdir="$wdir/$g/Stats";
    mkdir("$statdir/");
    $logdir="$opsdir/logs";
    system("/bin/cp -au $logdir/DpushM-${gn}*-2* $statdir/");
}

#----------------------------------------  Get current top process list
@topl = `$TOPCMD`;
chomp(@topl);
$cmdinx=10;
while (@topl != 0) {
    my $hdrline = shift(@topl);
    if ($hdrline =~ "PID USER") {
	$hdrline =~ s/^[ ]*//g;
	$hdrline =~ s/ [ ]*/ /g;
	my $inx=index($hdrline, " COMMAND");
	my @cols=split(' ', substr($hdrline,0,$inx));
	$cmdinx=@cols;
	last;
    }
}

for (my $i=0; $i<@topl; $i++) {
    $topl[$i] =~ s/^[ ]*//g;
    $topl[$i] =~ s/ [ ]*/ /g;
}


#----------------------------------------  Get a list of consumers
@consl = ();
if ($test_lsmp != 0) {
    @consl = `$SMDUMP -c`;
    while ( @consl && ! (shift(@consl) =~ "Consumer" ) ) {}
    shift(@consl);
    shift(@consl);
    chomp(@consl);
    for (my $i=0; $i<@consl; $i++) {
	$consl[$i] =~ s/^[ \t]+//;
	my @consarg = split(/[ \t]+/, $consl[$i]);
	$consl[$i] = $consarg[1];
    }
}

@RedFlags    = ();
@DoneProcs   = ();
@MissProcs   = ();
@MissProcSpi = ();
@SuppProcs   = ();
@SuppProcSpi = ();
@OKProcs     = ();
@OKProcSpi   = ();
@AllProcs    = ();

system("rm -f $sd/${g}_*_Mon*.html > /dev/null");

#------------------------------------- Get the procmgt PID
my @temp = ();
# my @temp=`ps -fu $user`;
# for (my $i=0; $i<@temp; $i++) {
#     $temp[$i] =~ s/^[ \t]+//;
# }
# my @gtemp=grep( /procmgt/, grep( /perl/, @temp) );
# my @tmp0=split( /[ \t]+/, $gtemp[0]);
# $MID=$tmp0[1];

$plistd="$opsdir/Process_List_$gn";
@proclist=`ls -1d $plistd/*`;
chomp(@proclist);

@munch=();
@partlist=`smlist | grep -v ' id ' | sed -e 's,  *, ,g' -e 's,^ ,,g' | cut -d ' ' -f 2`;
chomp(@partlist);
foreach $part (@partlist) {
    my @before=`cat $TD/Before_${g}_${part}.list`;
    chomp(@before);
    
    my @after=`cat $TD/After_${g}_${part}.list`;
    chomp(@after);

    my $total=0;
    for (my $i=0; $i<@after; $i++) {
	my @atemp=split(/[ \t]+/, $after[$i]);
	my $apid=$atemp[0];
	my $count=$atemp[1];
	for (my $j=0; $j<@before; $j++) {
	    my @btemp=split(/[ \t]+/, $before[$j]);
	    next if ($apid != $btemp[0]);
	    $count=$atemp[1] - $btemp[1];
	    last;
	}
	if ($apid == 0) {
	    $total=$count;
	    next;
	}
	push(@munch, "$apid $part $count $total");
    }
}

@SupportList=`cat $td/Support.proc`;
chomp(@SupportList);

#---------------------------------------  Get a list of current processes
@pslist=`ps -fu $user`;
chomp(@pslist);
shift(@pslist) if ( $pslist[0] =~ /PID  PPID/ );

#---------------------------------------  Loop over process list, build entries
open (FNODEMONHTML, ">$sd/${g}_${gLN}_Mon.html");
foreach $pdir ( @proclist ) {
    $PRC = $pdir;
    $PRC =~ s|/$||;
    $PRC =~ s|$plistd/||m;
    next if ( -e "$pdir/disable");

    #-----------------------------------  Get process list data
    my $PID="NA";
    $PID = readlink("$pdir/pid") if ( -l "$pdir/pid" );
    my $CWD="NA";
    $CWD = readlink("$pdir/cwd") if ( -l "$pdir/cwd" );
    my $OUT="NA";
    $OUT = readlink("$pdir/dmt_output") if ( -l "$pdir/dmt_output" );
    my $EXE="NA";
    $EXE = readlink("$pdir/exe") if ( -l "$pdir/exe" );
    my $HTM="NA";
    $HTM = readlink("$pdir/html_output") if ( -l "$pdir/html_output" );
    my $RST="NA";
    $RST = readlink("$pdir/restarts") if ( -l "$pdir/restarts" );
    my $TRN="NA";
    $TRN = readlink("$pdir/trend_output") if ( -l "$pdir/trend_output" );
    my $exename=$EXE;
    $exename =~ s|^.*/||g;

    my $EAT = "NA";
    my $NAT=-1;
    my $NRD=0;
    my $SID = $PID;
    if ("$PID" ne "NA") {
	for my $tmpi ( @pslist ) {
	    $tmpi =~ s/^[ \t]+//;
	    my @tmpl=split( /[ \t]+/, $tmpi );
	    $SID=$tmpl[1]  if ( $tmpl[2] == $PID );
	}

	push(@DoneProcs, $PID);

        for $pidl ( @munch ) {
            my @stat=split( /[ \t]+/, $pidl);
            next if ( $stat[0] != $SID );
            $NAT=$stat[2];
            $NRD=$stat[3];
            last;
        }

	#--------------------------------  negative count -> not reading
	if ($NAT > 20 || $NAT < 0) {
	    $EAT="RUNS";
	} else {
	    $EAT="IDLE";
	}
    }

    if ( -d "$dh/gds/monitor_reports/$PRC" ) {
	$HTM="$dw/monitor_reports/$PRC";
    } elsif ( -d "$dh/gds/monitor_reports/$EXE" ) {
	$HTM="$dw/gds/monitor_reports/$EXE";
    } else {
	$HTM="$dw/monitor_reports";
    }

    #-----------------------------------  Set RUN if process is in either
    #                                     the consumer or the top list
    $RUN="N";
    if ( "$PID" ne "NA" ) {
	$RUN = "Y" if ( grep( "$PID", @consl) != 0 );
	$RUN = "Y" if ( grep(/^$PID /, @topl ) != 0 );
	if ($SID != $PID) {
	    $RUN = "Y" if ( grep( "$SID", @consl) != 0 );
	    $RUN = "Y" if ( grep(/^$SID /, @topl ) != 0 );
	    push(@DoneProcs, $SID);
	}
    }

    $SUP="N";
    foreach my $proc (@SupportList) {
	if ( $exename eq $proc ) {
	    $SUP = "Y";
	    last;
	}
    }

    if ( -d "$dd/$PRC" ) {
	$DOC="$d2/$PRC";
    } elsif ( -d "$dd/$exename" ) {
	$DOC="$d2/$exename";
    } else {
	$DOC="$d2/UnDocumented";
    }

    if ( -f "$DOC/ref_url" ) {
	$DOC=`cat ref_url`;
	chomp($DOC);
    }

    if ( -e "$dd/$PRC/$PRC.synopsis" ) {
	$SYN=`cat $dd/$PRC/$PRC.synopsis`;
    } elsif ( -e "$dd/$exename/${exename}.synopsis" ) {
	$SYN=`cat $dd/$exename/${exename}.synopsis`;
    } else {
	$SYN=`cat $dd/UnDocumented/UnDocumented.synopsis`;
    }

    push(@AllProcs, "$PID $SID $EXE $PRC $RUN $SUP $RST $DOC $HTM $OUT $TRN $CWD");

    $COL="CCCCCC";
    $STATUS="Unknown";
    $BALL="yellow";
    $SUPPORT="n";
    if ( $RUN eq "N" ) {
	$COL="FF2211";
	$STATUS="<b>MISSING</b>";
	$BALL="red";
	push(@RedFlags, $PRC);
    } elsif ( $SUP eq "N" ) {
	$COL="0000DD";
	$STATUS="Running";
	$BALL="green";
    } else {
	$COL="AAAAAA";
	$STATUS="Support";
	$BALL="green";
	$SUPPORT="nS";
    }

    my @entry = ();
    my @entrySpi = ();
    push(@entry, "<br><br><b><font size=+1><a href=\"$HTM\">$PRC</a></b>&nbsp;&nbsp;[<a href=\"$DOC\">$exename</a>]</font>&nbsp;&nbsp;&nbsp;&nbsp;\n");
    push(@entry, $SYN);
    push(@entry, "<br>[Resurrectable: <b>$RST</b> times]&nbsp;&nbsp;[<a href=\"$HTM\">Results</a>]&nbsp;&nbsp;[<a href=\"$DOC\">Description</a>]&nbsp;&nbsp;[Trend Output: $TRN]&nbsp;&nbsp;\n");
    push(@entry, "<br>[DMT Output: $OUT]&nbsp;&nbsp;[Instance PID: $PID]&nbsp;&nbsp;[Parent PID: $SID]&nbsp;&nbsp;[Configured from: $CWD]&nbsp;&nbsp;\n");

    @temp = grep( /$SID/, grep /$exename/, @topl );
    if (@temp) {
	my @tmp0 = split(/[ \t]+/, $temp[0] );
	push(@entry, "<br>PID: <b>$tmp0[0]</b>  |  USER: <b>$tmp0[1]</b>  |  CPU: <b>$tmp0[9]</b>  |  SIZE: <b>$tmp0[5]</b>  RES: <b>$tmp0[6]</b> | TIME: <b>$tmp0[8]</b>\n");
    }

    if ( $EAT eq "IDLE") {
	push(@entry, "<br><font color=\"#FF0000\">IDLE : Consumed  $NAT/$NRD frames. </font>\n");
	$IsSluggish="YES";
	$COL="FF0000";
	$STATUS="<b>IDLE</b>";
	$BALL="red";
	push(@RedFlags, $PRC);
    } elsif ( $EAT eq "RUNS" && $NAT >= 0) {
	push(@entry, "<br>Running : Consumed  $NAT/$NRD  frames.\n");
    }

    push(@entrySpi, "<font size=+1><b><a name=\"$PRC\" href=\"$HTM\">$PRC</a></b>&nbsp;[<a href=\"$DOC\">$exename</a>]</font>&nbsp;&nbsp;\n");

#---------------------------------------- Add entries to the running list
    if ( $RUN eq "N" ) {
	push(@MissProcs,   @entry);  
	push(@MissProcSpi, @entrySpi);  
    } elsif ( $SUP eq "N" ) {
	push(@OKProcs,     @entry);
	push(@OKProcSpi,   @entrySpi);
    } else {
	push(@SuppProcs,   @entry);
	push(@SuppProcSpi, @entrySpi);
    }

    if ( $SUPPORT eq "n" ) {
	print(FNODEMONHTML "<!- $PRC-->",
	      "<tr><td><img SRC=\"pix/ball_$BALL.gif\"></td>",
	          "<td><a href=\"$HTM\"><b>$PRC</b></a></td>",
	          "<td><font color=\"#$COL\">$STATUS</font></td>",
	          "<td><a href=\"$DOC\">$exename</a></td>",
	          "<td><a href=\"$wurl/${g}_spi.html\">$g</a></td>",
	          "<td>$gLN</td>",
	      "</tr>\n");
    }
}

#--------------------------------------- Find and fill in rogue processes
@RogueProcs = ();
@RogueSpi   = ();
for (my $i=0; $i<@consl; $i++) {
    my $PID = $consl[$i];
    next if ( grep( /$PID/, @DoneProcs) != 0);
    next if ( ! -d "/proc/$PID" );

    my @temp = grep(/^$PID /, @topl);
    next if (@temp == 0);
    my @tmp0 = split(' ', $temp[0]);

    my $EXE=$tmp0[$cmdinx];

    if ( -d "$dh/gds/monitor_reports/$EXE" ) {
	$HTM="$dw/gds/monitor_reports/$EXE";
    } else {
	$HTM="$dw/gds/monitor_reports/";
    }

    if ( -d "$dd/$EXE" ) {
	$DOC="$d2/$EXE";
    } else {
	$DOC="$d2/UnDocumented";
    }

    if ( -e "$dd/$EXE/${EXE}.synopsis" ) {
	$SYN=`cat $dd/$EXE/${EXE}.synopsis`;
    } else {
	$SYN=`cat $dd/UnDocumented/UnDocumented.synopsis`;
    }

    push(@AllProcs, "$PID $EXE $DOC $HTM");

    push(@RogueProcs, "<br><br><b><font size=+1><a name=\"$EXE\" href=\"$HTM}\">$EXE</a></b></font>&nbsp;&nbsp;&nbsp;&nbsp;\n");
    push(@RogueProcs, $SYN);
    push(@RogueProcs, "&nbsp;&nbsp;[<a href=\"$DOC\">Description</a>]\n");

    push(@RogueProcs, "<br>PID: <b>$tmp0[0]</b>  |  USER: <b>$tmp0[1]</b>  |  CPU: <b>$tmp0[9]</b>  |  SIZE: <b>$tmp0[5]</b>  RES: <b>$tmp0[6]</b> | TIME: <b>$tmp0[8]</b>\n");

    push(@RogueSpi, "<b><font size=+1><a href=\"$HTM\">$EXE</a></b>&nbsp;[<a href=\"$DOC\">Doc</a>]</font>&nbsp;&nbsp;\n");

    $COL="FF0099";
    $STATUS="<b>Testing</b>";
    $BALL="purple";

    print(FNODEMONHTML 
	  "<!-- $EXE -->",
	  "<tr><td><img SRC=\"pix/ball_$BALL.gif\"></td>",
	      "<td><a href=\"$HTM\"><b>$EXE</b></a></td>",
	      "<td><font color=\"#$COL\">$STATUS</font></td>",
	      "<td><a href=\"$DOC\">$EXE</a></td>", 
	      "<td><a href=\"$wurl/${g}_spi.html\">${g}</a></td>", 
	      "<td>$gLN</td>",
	  "</tr>\n");
}
close(FNODEMONHTML);

#-----------------------------------------  Make the SpiMaker file
open(fSPIMAKER, ">$sd/${g}_spiMaker.html");

if ( $IsSluggish eq "YES" ) {
    print(fSPIMAKER "<font color=\"#FF0000\"><b><blink>IDLE processes exist</blink></b></font><br>\n");
}

if ( @OKProcSpi ) {
    print(fSPIMAKER "<font color=\"#0000FF\"><font size=+1>ProcessManaged:</font>\n");
    print(fSPIMAKER @OKProcSpi);
    print(fSPIMAKER "</font><br>\n");
}

if ( @MissProcSpi ) {
    print(fSPIMAKER "<font color=\"#FF0000\"><font size=+1>MissingMonitor:</font>\n");
    print(fSPIMAKER @MissProcSpi);
    print(fSPIMAKER "</font><br>\n");
}

if ( @RogueSpi ) {
    print(fSPIMAKER "<font color=\"#555555\"><font size=+1>DevelopingMons:</font>\n");
    print(fSPIMAKER @RogueSpi);
    print(fSPIMAKER "</font><br>\n");
}

if ( @SuppProcSpi ) {
    print(fSPIMAKER "<font color=\"#00FF44\"><font size=+1>SupportProcess:</font>\n");
    print(fSPIMAKER @SuppProcSpi);
    print(fSPIMAKER "</font><br>\n");
}
close(fSPIMAKER);

system("cp $sd/${g}_spiMaker.html $wdir/${g}_spiMaker.html");
system("cp $sd/${g}_*_Mon.html $wdir");

$redList=join(' ', @RedFlags);
system("echo $redList > $wdir/badMon_${g}.tmp");
system("mv $wdir/badMon_${g}.tmp $wdir/badMon_${g}.txt");

#----------------------------------------  Generate time-stamps
$stamp_gps=`$TNOW`;   chomp($stamp_gps);
$stamp_lcl=`date`;    chomp($stamp_lcl);
$stamp_utc=`date -u`; chomp($stamp_utc);

#----------------------------------------  Open the temporary spi file
$spitFile="$sd/${g}_spiT.html";
open (FSPITFILE, ">$spitFile");
print(FSPITFILE `sed -e s/COMPNAME/$g/g -e s/LIGO/$gLN/g $td/nose_base.html`);
print(FSPITFILE "<br><br>\n");
print(FSPITFILE "<table BORDER COLS=4 WIDTH=\"100%\">\n");
print(FSPITFILE "<tr><td>Created at</td>\n");
print(FSPITFILE "<td><b><font color=\"#FF0000\"> $stamp_gps </td>\n");
print(FSPITFILE "<td><b><font color=\"#FF00DD\"> $stamp_lcl </td>\n");
print(FSPITFILE "<td><b><font color=\"#0000FF\"> $stamp_utc </td></tr>\n");
print(FSPITFILE "</table><br>\n");

#----------------------------------------  Processing for master node.
my $othdom="ligo-la";
my $othobs="LLO";
if ($gip eq $othdom) {
    $othdom="ligo-wa";
    $othobs="LHO";
}
if ( $ms eq "M" ) {
    print("building master page\n") if ($debug);

    #------------------------------------  Get data pages from the other domain
    my @othhosts = grep(/$othdom/, @hostlist);
    chomp(@othhosts);

    for my $hostline (@othhosts) {
	my @hostargs = split(/[ \t]+/, $hostline);
	my $hosturl = $hostargs[4];
	my $hostID  = $hostargs[1];

	my $host_spi="${hostID}_spi.html";
	unlink("$wdir/${host_spi}") if ( -f "$wdir/${host_spi}");
	system("$WGET -P $wdir $hosturl/${host_spi}");

	
	my $host_spiMk="${hostID}_spiMaker.html";
	unlink("$sd/${host_spiMk}") if (-f "$sd/${host_spiMk}");
	system("$WGET -P $sd $hosturl/${host_spiMk}");

	my $host_Mon="${hostID}_${othobs}_Mon.html";

	unlink("$sd/${host_Mon}") if (-f "$sd/${host_Mon}");
	system("$WGET -P $sd $hosturl/${host_Mon}");
    }

    #------------------------------------  Generate the node-by-node page
    writeSpiC();

    #------------------------------------ Create monitor indexed files
    foreach my $IFO ( "_LLO", "_LHO", "" ) {
	writeSpiM( "$IFO" );
	system("cp $wdir/spiM${IFO}.html $wdir/spi${IFO}.html");
    }

    #------------------------------------  Generate json list
    my @lclhosts = grep(/$gip/, @hostlist);
    chomp(@lclhosts);

    my $redLine = "";
    for my $hostline (@lclhosts) {
	my @hostargs = split(/[ \t]+/, $hostline);
	my $hostID  = $hostargs[1];
	my $hostBad = `cat $wdir/badMon_${hostID}.txt`;
	chomp($hostBad);
	if (length($hostBad) != 0) {
	    $redLine = "$redLine " if (length($redLine) != 0);
	    $redLine = "$redLine$hostBad";
	}
    }
    if (length($redLine) != 0) {
	system("sed -e 's,MONITORS,$redLine,g' -e 's,GPS_TIME,$stamp_gps,g' -e 's,SPI_PAGE,$wurl/spiM_${gLN}.html,g' $td/monitor-status-bad.json > $wdir/monitor-status.json");
    } else {
	system("sed -e 's,GPS_TIME,$stamp_gps,g' -e 's,SPI_PAGE,$wurl/spiM_${gLN}.html,g' $td/monitor-status-good.json > $wdir/monitor-status.json");
    }

#----------------------------------------  Files for local master node.
}

print(FSPITFILE "<tt>\n");
if ( @OKProcs ) {
    print(FSPITFILE "<br><hr SIZE=1 WIDTH=\"80%\">\n");
    print(FSPITFILE "<br><font color=\"#0000FF\"><font size=+2>",
	             "<b>Monitors running under process manager:</b></font>\n");
    print(FSPITFILE @OKProcs);
    print(FSPITFILE "</font><br>\n");
}

if ( @MissProcs ) {
    print(FSPITFILE "<br><hr SIZE=1 WIDTH=\"80%\">\n");
    print(FSPITFILE "<br><font color=\"#FF0000\"><font size=+2>",
 	             "<b>Missing monitors:</b></font>\n");
    print(FSPITFILE @MissProcs);
    print(FSPITFILE "</font><br>\n");
}

if ( @RogueProcs ) {
    print(FSPITFILE "<br><hr SIZE=1 WIDTH=\"80%\">\n");
    print(FSPITFILE "<br><font color=\"#555555\"><font size=+2>",
	             "<b>Monitors in testing phase:</b></font>\n");
    print(FSPITFILE @RogueProcs);
    print(FSPITFILE "</font><br>\n");
}

if ( @SuppProcs ) {
    print(FSPITFILE "<br><hr SIZE=1 WIDTH=\"80%\">\n");
    print(FSPITFILE "<br><font color=\"#00FF44\"><font size=+2>",
	             "<b>Support processes:</b></font>\n");
    print(FSPITFILE @SuppProcs);
    print(FSPITFILE "</font><br>\n");
}

print(FSPITFILE "</tt>\n");
print(FSPITFILE "</font><p><hr SIZE=1 WIDTH=\"80%\"><br>", 
                "<br><font color=\"#000000\">\n");

#######################################
# Making the tail of the page         #
#######################################

$T_stop=`$TNOW`;
$T_dur=$T_stop - $T_start;

system("cp $sd/${g}_spiMaker.html $wdir/${g}_spiMaker.html");

print(FSPITFILE `cat $td/tail.html`);
print(FSPITFILE `date`);
print(FSPITFILE  "PM</font></b>\n");
my $statl = `grep "<br>CPU&nbsp;states:&nbsp;" $wdir/$g/TOPinfo.html`;
chomp($statl);
my $rr = "N/A";
if ( "x$statl" ne "x") {
    $statl =~ s/\&nbsp\;/ /g;
    @temp=split(/ /, $statl);
    $temp[2] =~ s/\(.*$//g;
    $rr =100.0 - $temp[2];
}

# Add tag line (used to check up to date status)
print(FSPITFILE "<font color=\"#CCCCCC\"><font size=-2>\n");
my @tnow=localtime();
$tnow[2] -=  2 if ("$gip" eq "ligo-la");
$tnow[2] += 24 if ($tnow[2] < 0);
print(FSPITFILE "TI $tnow[2] $tnow[1] $rr\n");
print(FSPITFILE "</font></font>\n");
print(FSPITFILE "<br>Script run time: $T_dur sec<br>\n");
print(FSPITFILE `cat $td/refresh.html`);
print(FSPITFILE "</body>\n");
print(FSPITFILE "</html>\n");

# Place html page to public directory

close (FSPITFILE);
system("mv $sd/${g}_spiT.html $wdir/${g}_spiT.html");
rename("$wdir/${g}_spiT.html", "$wdir/${g}_spi.html");

# Mail out error messages to listed experts

#if -e $sd/${g}_Errors.log) {
#foreach my $list ( `cat $wdir/MailErrorsTo.list` ) 
# cat $sd/${g}_Errors.log | mailx -s "Error message from ${g}" $list
#end
#}

#======================================  Write the SpiC.html file
#
#  The spiC page is a global spi page organized by node name. It is 
#  built by concatinating the $sd/${comp}_spiMaker.html files with 
#  appropriate header and trailer lines.
#
sub writeSpiC {
    #----------------------------------- Create the file. Copy header with
    #                                    appropriate refresh address.
    open(GSPITFILE, ">$sd/spiT.html");
    print(GSPITFILE `sed -e "s/URL=spi.html/URL=spiC.html/g" $td/spiHeader.html`);

    #----------------------------------- Add timestamp.
    print(GSPITFILE "<table BORDER COLS=4 WIDTH=\"100%\">\n");
    print(GSPITFILE "<tr><td>Created at</td>\n");
    print(GSPITFILE "<td><b><font color=\"#FFDD00\"> $stamp_gps </td>\n");
    print(GSPITFILE "<td><b><font color=\"#FF00DD\"> $stamp_lcl </td>\n");
    print(GSPITFILE "<td><b><font color=\"#0000FF\"> $stamp_utc </td></tr>\n");
    print(GSPITFILE "</table><br>\n");

    #-----------------------------------  Loop over sites
    foreach $OBS ( "LHO", "LLO", "CIT", "MIT" ) {

	#-------------------------------  Loop over nodes (select current site)
	for (my $k=0; $k<@hostlist; $k++) {
	    my @host=split(/[ \t]+/, $hostlist[$k]);
	    next if ( "$host[3]" ne "$OBS" );
	    my $comp = $host[1];
	    my $spiMk="${comp}_spiMaker.html";
	    if ( ! -f "$sd/$spiMk" ) {
		if (-f "$wdir/$spiMk" ) {
		    system("cp -au $wdir/$spiMk $sd/");
		} else {
		    system("touch $sd/$spiMk");
		}
	    }
	    my $spiMon="${comp}_${OBS}_Mon.html";
	    if ( ! -f "$sd/$spiMon" ) {
		if (-f "$wdir/$spiMon" ) {
		    system("cp -au $wdir/$spiMon $sd/");
		} else {
		    system("touch $sd/$spiMon");
		}
	    }
	    my $wht  = $host[4];
	    print(GSPITFILE "<font color=\"#000088\"><font size=+1>");
	    print(GSPITFILE "<a href=\"$wht/${comp}_spi.html\">$comp</a>");
	    print(GSPITFILE "</font></font><br>\n");
	    print(GSPITFILE `cat $sd/$spiMk`);
	    print(GSPITFILE "<hr SIZE=1 WIDTH=\"75%\">\n");
	}
    }

    #-----------------------------------  Add trailer and close file
    print(GSPITFILE `cat $td/spiTrailer.html`);
    close(GSPITFILE);

    #-----------------------------------  Replace file in target directory
    system("mv $sd/spiT.html $wdir/spiT.html");
    rename("$wdir/spiT.html", "$wdir/spiC.html");
}

#======================================  Write a SpiM[_<ifo>].html file
#
#  The spiM page is a global spi page organized by process. It is 
#  built by concatinating the $sd/cache/*${ifo}_Mon.html files with 
#  appropriate header and trailer lines.
#
sub writeSpiM {
    my $IFO = $_[0];
    open(SPIMTIFO, ">$sd/spiMT${IFO}.html");
    print(SPIMTIFO `cat $td/spiMHeader${IFO}.html`);
    print(SPIMTIFO "<table border=\"0\" COLS=4 WIDTH=\"100%\">\n");
    print(SPIMTIFO "<tr><td><font color=\"#333333\">Created at: </font></td>\n");
    print(SPIMTIFO "<td><b><font color=\"#DDDD22\"> $stamp_gps </td>\n");
    print(SPIMTIFO "<td><b><font color=\"#DD00DD\"> $stamp_lcl </td>\n");
    print(SPIMTIFO "<td><b><font color=\"#33FF33\"> $stamp_utc </td></tr>\n");
    print(SPIMTIFO "</table><br>\n");
    print(SPIMTIFO "<table BORDER=\"0\" bgcolor=\"EEEEEE\" 
                           bordercolor=\"#FFFFFF\" RULES=ROWS FRAME=HSIDES 
                           COLS=6 WIDTH=\"100%\">\n");
    print(SPIMTIFO 
	  "<tr><th>?OK?</th><th>Monitor instance</th>",
	      "<th>Status</th>",
	      "<th>Description of monitor class</th>",
	      "<th>Host and process details</th>",
	      "<th>Site</th>", 
	  "</tr>",
	  "<tr><td>&nbsp;</td>",
	      "<td></td>",
	      "<td></td>",
	      "<td></td>",
	      "<td></td>",
	      "<td></td>",
	  "</tr>\n");
    print(SPIMTIFO `cat $sd/*${IFO}_Mon.html | sort -f`);
    print(SPIMTIFO "</table><br>\n");
#    print(SPIMTIFO `cat $sd/Errors.log`) if ( -e "$sd/Errors.log" );
    print(SPIMTIFO `tail $sd/Errors.log`) if ( -e "$sd/Errors.log" );
    print(SPIMTIFO `cat $td/spiMTrailer.html`);
    close(SPIMTIFO);
    system("mv $sd/spiMT${IFO}.html $wdir/spiMT${IFO}.html");
    rename("$wdir/spiMT${IFO}.html", "$wdir/spiM${IFO}.html");
}
