#!/bin/bash

#
# -- Macros
#
#    threshold compares the first argument to the second. If first is less
#    than or equal to the second, the third argument is echoed. Otherwise 
#    fourth argument is echoed (if it exists)
threshold() {
  if [ $1 -le $2 ]; then
    echo $3
  elif [ $# -ge $4 ]; then
    echo $4
  fi
}

#######################################
#        Initialize the stuff         #
#######################################
prefix=/usr
exec_prefix=${prefix}
datarootdir=${prefix}/share
sd=$HOME/dmtspi/cache
td=${datarootdir}/gds/Spi

mkdir -p $sd
TD="/tmp/"

# Identify machine
gn=`uname -n`
g=`cat $td/HOST.lookup | awk '{if($1==gn) print $2}' gn=$gn`
gip=`cat $td/HOST.lookup | awk '{if($1==gn) print $3}' gn=$gn`
gLN=`cat $td/HOST.lookup | awk '{if($1==gn) print $4}' gn=$gn`

# Define remote directories

if [ "$gip" = "ligo-wa" ]; then
  dh="/gds-h1/dmt/html"
  dw="https://marble.ligo-wa.caltech.edu/dmt"
elif [ "$gip" == "ligo-la" ]; then
  dh="/gds/dmt/html"
  dw="https://llodmt.ligo-la.caltech.edu/dmt"
else
  dh="$HOME/public_html/gds"
  dw="http://www.ligo.caltech.edu/~$USER/gds"
fi

dd="$dh/dmt/Monitors"
d2="$dw/dmt/Monitors"

mkdir -p $dd/$g

#######################################
#  Look for aged files                #
#######################################

touch $sd/Errors.log
MaxAge=1000

for F in `ls $sd | grep spi | grep 'html$'`; do
  touch /tmp/timetest 
  NOW=`stat -c %Y /tmp/timetest`
  TCH=`stat -c %Y $sd/$F`  
  AGE=`expr $NOW - $TCH`
  STS=`threshold $AGE $MaxAge NEW OLD`

  if [ "$STS" = "OLD" ]; then
    sed -e "s,REPLACETHIS,$sd/$F,g" $td/ErrorCondition.html > $sd/$F
    stamp=`date`
    cat >> $sd/Errors.log <<EOF
<font color="#FF0000">
<br>$stamp<br>Obsolete file was found ! Something stopped running!<br>
$sd/$F
<br>After fixing the problem, you must remove the $sd/Errors.log file!<br>
</font>
EOF

  fi
done

for G in `ls $dd/*.html | grep spi`; do
  touch $TD/timetest 
  NOW=`stat -c %Y $TD/timetest`
  TCH=`stat -c %Y $G`  
  AGE=`expr $NOW - $TCH`
  STS=`threshold $AGE $MaxAge NEW OLD`

  if [ "$STS" = "OLD" ]; then
    sed -e "s,REPLACETHIS,$G,g" $td/ErrorCondition.html > $G
    cat >> $sd/Errors.log <<EOF
<font color="#FF0000">
<br>Obsolete file was found ! Something stopped running!<br>
$G
<br>After fixing the problem, you must remove the $sd/Errors.log file!<br>
</font>
EOF
  fi
done
