#! /usr/bin/perl -w
#
#  Prune.pl
#
#  Perl script to prune old data directories
#
# --> Initialization
$|=1;
#
# --> Parse command line
$syntax=0;
$debug=0;
$rc=1;
$max_age=1200000;
$directory="";
print("Parsing arguments\n") if ($debug);
for (my $inx=0; $inx < @ARGV; $inx++) {
    my $Argi = $ARGV[$inx];
    print("Argument $inx: $Argi\n") if ($debug);
    if ($Argi eq "--maximum-age" || $Argi eq "-a") {
	$inx++;
	if ($inx >= @ARGV) {
	    print("Missing maximum age argument\n");
	    $syntax=1;
	    last;
	}
	$max_age=$ARGV[$inx];
    } elsif ($Argi eq "--root-directory" || $Argi eq "-r") {
	$inx++;
	if ($inx >= @ARGV) {
	    print("Missing directory argument\n");
	    $syntax=1;
	    last;
	}
	$directory=$ARGV[$inx];
    } elsif ($Argi eq "--debug") {
	$debug=1;
	$rc=0;
    } elsif ($Argi eq "--help") {
	$syntax=1;
	$rc=0;
    } else {
	print("Invalid argument: $Argi\n");
	$syntax=1;
	last;
    }
}
#
#  -- > Check that specified directory
if ( $directory eq "" ) {
    print("Root directory not specified\n");
    $syntax=1;
    $rc=2;
} elsif ( ! -d $directory ) {
    print("Specified root ($directory) is not a directory\n");
    $syntax=1;
    $rc=2;    
}

if ($syntax) {
    print("The prune command removes out-of-date GPS sub-directories\n\n");
    print("Command syntax:\n");
    print("prune {--root-directory | -r} <root-directory> [--help]\n");
    print("      [--maximum-age | -a <maximum-age> [--debug]\n");
    exit($rc);
}
#
# --> Get the current GPS
$gps_now=`when 0 %s`;
printf("current time: $gps_now\n") if ($debug);
#
# --> open the specified directory
opendir(ROOTDIR, $directory);
while (defined(my $dir = readdir(ROOTDIR) ) ) {
    my $fdir="$directory/$dir";
    print("Processing: $fdir\n") if ($debug);
    if ( -d $fdir && $dir=~/[HLV]-([A-Za-z0-9_]+)-([0-9]+)/ ) {
	my $gpsfrag="$2";
	my $gps_end=$gpsfrag+1;
        if ( -f "$fdir/.no_prune" ) {
	    print("Pruning disabled on directory: $fdir\n");
	    next;
	}
	while ($gps_end < 500000000 ) {
	    $gps_end *= 10;
	}
	if ($gps_end + $max_age < $gps_now) {
	    print("Remove directory $fdir ...");
	    if ($debug) {
		print("Suppressed by --debug\n");
	    } else {
		system("rm -r $fdir");
		print("Done\n");
	    }
	}
    } elsif ($debug) {
	printf("Directory: $fdir skipped.\n") if ($debug);
    }
}
closedir(ROOTDIR);
exit(0);
