#!  -f
#
#   Get the DMT home directory
#
set svargs="$argv"
set platform = `uname -s`_`uname -m`
if (! $?DMTHOME)    setenv DMTHOME    
if (! $?ROOTSYS)    setenv ROOTSYS    /usr
if (! $?DMTVERSION) setenv DMTVERSION usr
if ( -d $DMTHOME/$platform ) setenv DMTHOME $DMTHOME/$platform
#
#   Set up the DMT environment variables
#
#   DMTNAMESERVER Name server node:port
#   DMTPARS       DMT parameter directory
#   GDSAPACHE     Directory containing gds web pages [when available]
#   LIGOSITE      LIGO site name (LHO, LLO, CIT, ...)
#   LIGONDSIP     Default NDS IP address
#   LIGOMCASTIP   Default Multicast address
#
setenv DMTPARS       ${HOME}/pars
setenv LIGOMCASTIP   225.0.0.1
#
#  Set site dependent parameters based on domain name
#
if (-f /etc/resolv.conf && `grep -c domain /etc/resolv.conf`) then
    set domain = `grep domain /etc/resolv.conf`
    if ("$domain" == "") set domain = domain unknown
    set domain = $domain[2]
    if ("$domain" == "ligo.caltech.edu") then
	setenv LIGOSITE      CIT
    else if ("$domain" == "ligo-wa.caltech.edu") then
	setenv LIGOSITE      LHO
    else if ("$domain" == "ligo-la.caltech.edu") then
	setenv LIGOSITE      LLO
    else if ("$domain" == "cds.ligo-la.caltech.edu") then
	setenv LIGOSITE      LLO
    else if ("$domain" == "mit.edu") then
	setenv LIGOSITE      MIT
    else
	setenv LIGOSITE      LIGO
    endif
    unset domain
else
    setenv LIGOSITE          LIGO
endif
#
#   Override the variables if necessary
#
set host    = `uname -n`
set envfile = $DMTHOME/$DMTVERSION/bin/envars.$host
if (-f $envfile) source $envfile
unset envfile
#
#   Derive site dependent information
#
if (! $?LIGOSMPART) setenv LIGOSMPART  ${LIGOSITE}_Online
if ($?LIGOSMPART && ! $?DMTINPUT) setenv DMTINPUT /online/${LIGOSMPART}
#
if ("$LIGOSITE" == "LHO") then
    if (! $?GDSAPACHE)     setenv GDSAPACHE     /src/cds/lho/web/gds
    if (! $?DMTIFOS)       setenv DMTIFOS       "H0 H1 H2"
    if (! $?DMTNAMESERVER) setenv DMTNAMESERVER gdsdmt0:23456
    if (! $?LIGONDSIP)     setenv LIGONDSIP     fb1
else if  ("$LIGOSITE" == "LLO") then
    if (! $?GDSAPACHE)    setenv GDSAPACHE /import/space/apacheFiles/htdocs/gds
    if (! $?DMTIFOS)       setenv DMTIFOS       "L0 L1"
    if (! $?DMTNAMESERVER) setenv DMTNAMESERVER clearwater:23456
    if (! $?LIGONDSIP)     setenv LIGONDSIP     fb1
else if  ("$LIGOSITE" == "CIT") then
    if (! $?DMTNAMESERVER) setenv DMTNAMESERVER alterf:23456
    if (! $?DMTIFOS)       setenv DMTIFOS       "H0 H1 H2 L0 L1"
else if  ("$LIGOSITE" == "MIT") then
    if (! $?DMTNAMESERVER) setenv DMTNAMESERVER lancelot:23456
    if (! $?DMTIFOS)       setenv DMTIFOS       "H0 H1 H2 L0 L1"
endif
#
#   Set up the compilation environment variables
#
#   TARGET    destination for compiled components
#
if (! $?TMPDIR) setenv TMPDIR /tmp
if (! $?TARGET) setenv TARGET $HOME
#
#  Binary, library and manual path setup
#
if (! $?MANPATH && `uname -s` != "Linux") setenv MANPATH /usr/local/man:/usr/man
if (! $?LD_LIBRARY_PATH) setenv LD_LIBRARY_PATH /usr/local/lib
#
#--> Set up ligo tools
if ($?LIGOTOOLS) then
else if (-x /opt/ligotools/ ) then
    setenv LIGOTOOLS /opt/ligotools
else if (-x /ligoapps/ligotools/ ) then
    setenv LIGOTOOLS /ligoapps/ligotools
else if (-x /ligotools/ ) then
    setenv LIGOTOOLS /ligotools
endif
if ($?LIGOTOOLS) then
    if (-x $LIGOTOOLS/bin/use_ligotools) eval `$LIGOTOOLS/bin/use_ligotools`
endif
#
#--> One argument at a time for csh
set argv=$ROOTSYS; source $DMTHOME/$DMTVERSION/bin/addpath $ROOTSYS
setenv LD_LIBRARY_PATH ::${LD_LIBRARY_PATH}
#
#--> Add the DMT libraries in front!
set argv=$DMTHOME/$DMTVERSION; source $DMTHOME/$DMTVERSION/bin/addpath $DMTHOME/$DMTVERSION
#
#--> Done
set argv="$svargs"
unset svargs
