#! /bin/bash
#
#   Define setenv for envars, etc
#
setenv() {
export $1="$2"
}
#
#   Get the DMT home directory
#
# set -v
#
platform=`uname -s`_`uname -m`
export DMTHOME=${DMTHOME:-""}
export ROOTSYS=${ROOTSYS:-"/usr"}
export DMTVERSION=${DMTVERSION:-"usr"}
if [ -d $DMTHOME/$platform ]; then
    export DMTHOME="$DMTHOME/$platform"
fi
#
#   Set up the DMT environment variables
#
#   DMTNAMESERVER Name server node:port
#   DMTPARS       DMT parameter directory
#   GDSAPACHE     Directory containing gds web pages [when available]
#   LIGOSITE      LIGO site name (LHO, LLO, CIT, ...)
#   LIGONDSIP     Default NDS IP address
#   LIGOMCASTIP   Default Multicast address
#
export DMTIFOS="H0 H1 H2 L0 L1"
export DMTPARS=${DMTHOME}/pars
export LIGOMCASTIP=225.0.0.1
#
#  Set site dependent parameters based on domain name
#
if [ -e /etc/resolv.conf ] ; then
    domain=`grep domain /etc/resolv.conf | sed -e 's,^domain,,' -e 's,^name,,' -e 's, ,,g'`
    if [ "$domain" = "ligo.caltech.edu" ] ; then 
	export LIGOSITE=CIT
    elif [ "$domain" = "ligo-wa.caltech.edu" ] ; then
	export LIGOSITE=LHO
    elif [ "$domain" = "cds.ligo-la.caltech.edu" ] ; then
	export LIGOSITE=LLO
    elif [ "$domain" = "ligo-la.caltech.edu" ] ; then
	export LIGOSITE=LLO
    elif [ "$domain" = "ligo.mit.edu" ] ; then
	export LIGOSITE=MIT
    else
	export LIGOSITE=LIGO
    fi
else
    export LIGOSITE=LIGO
fi
#
#---> Set parameters based on site
if [ "$LIGOSITE" == "LHO" ] ; then
    export DMTIFOS="H0 H1 H2"
    export DMTNAMESERVER=gdsdmt0:23456
    export GDSAPACHE=/src/cds/lho/web/gds
    export LIGONDSIP=fb1
elif [ "$LIGOSITE" == "LLO" ] ; then
    export DMTIFOS="L0 L1"
    export DMTNAMESERVER=clearwater:23456
    export GDSAPACHE=/import/space/apacheFiles/htdocs/gds
    export LIGONDSIP=fb1
elif [ "$LIGOSITE" == "CIT" ] ; then 
    export DMTNAMESERVER=atlas:23456 
    export DMTIFOS="H0 H1 H2 L0 L1"
else
    export DMTIFOS="H0 H1 H2 L0 L1"
fi
#
#   Derive default partition from site name
#
export LIGOSMPART="${LIGOSITE}_Online"
#
#   Override the variables if necessary
#
host=`uname -n`
envfile=$DMTHOME/$DMTVERSION/bin/envars.$host
if [ -e $envfile ] ; then 
    source $envfile 
fi
#
#    Derive default input source.
#
if [ "${LIGOSMPART}x" != "x" ] ; then 
    export DMTINPUT=${DMTINPUT:-"/online/${LIGOSMPART}"}
fi
#
#   Set up the compilation environment variables
#
#   TARGET    destination for compiled components
#
export TMPDIR=${TMPDIR:-"/tmp"}
export TARGET=${TARGET:-$HOME}
#
#  Binary, library and manual path setup
#
export MANPATH=${MANPATH:-"/usr/local/man:/usr/man"}
export LD_LIBRARY_PATH=${LD_LIBRARY_PATH:-""}
#
#--> Set up ligo tools
if [ -z $LIGOTOOLS ] ; then 
    if [ -d /opt/ligotools/ ] ; then
	export LIGOTOOLS=/opt/ligotools 
    elif [ -d /ligoapps/ligotools/ ] ; then
	export LIGOTOOLS=/ligoapps/ligotools
    elif [ -d /ligotools/ ] ; then
	export LIGOTOOLS=/ligotools
    fi ;
fi
if [ -x $LIGOTOOLS/bin/use_ligotools ] ; then 
    eval `$LIGOTOOLS/bin/use_ligotools` 
fi
#
#--> One argument at a time for csh
source $DMTHOME/$DMTVERSION/bin/addpathb $ROOTSYS
LD_LIBRARY_PATH=${LD_LIBRARY_PATH}::
export LD_LIBRARY_PATH
#
#--> Add the DMT libraries in front!
source $DMTHOME/$DMTVERSION/bin/addpathb $DMTHOME/$DMTVERSION 
